#$Id: ModDM.pm,v 1.11 2007/01/11 02:47:47 uhtu Exp $

# Copyright (c) 2003-2007 Hugh Kennedy <kennedyh@engin.umich.edu>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModDM;

# dungeon-master module for omnicron. "roll 3 d5"
# "roll <n> d<m>"   -> roll n m-sided dice
# "roll soc"        -> roll 2 6-sided dice + pirate die
# "roll percentile" -> roll d% via two d10
# "roll roshambo"   -> break a two way tie
# "roll rps25 person1 person2" -> break a two way tie stupidly
#

@ModDM::ISA = qw(OmniMod);

use strict;
my $MAX_COUNT = 10;
my $MAX_SIDES = 100;
my $CARTMAN   = 0.000001;

# the irc colorspec, in all its horrible glory:
# http://www.felmlee.com/bodebot/docs/colors.txt

# primes are pirates, obviously. behold, the hash ARRGH:
my %ARRGH = (
             1 => "\cc2blue\cc",
             2 => "ARRGH",
             3 => "ARRGH",
             4 => "\cc8yellow\cc",
             5 => "ARRGH",
             6 => "\cc3green\cc",
            );

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $self  = $class->SUPER::new();
  bless ($self, $class);          # reconsecrate
  return $self;
}

sub help {
  if ( defined wantarray ) { # do we have help?
      return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  $irc->notice( $nick, "[roll <n> d<m>]  ->  roll n m-sided dice" );
  $irc->notice( $nick, "[roll soc]  ->  roll 2 6-sided dice + pirate die" );
  $irc->notice( $nick, "[roll percentile] -> roll d% via two d10" );
  $irc->notice( $nick, "[roll roshambo] -> break a two way tie" );
  $irc->notice( $nick, "[rps25 <user1> <user2>] -> rock paper scissors 25" );
}

sub name() {
  my $this = shift;
  return "dungeonmaster";
}

sub handlePrivateMessage {
  my ($this, $irc, $nick, $message, $event, $conn ) = @_;
  return handleMessage($this, $irc, $nick, $nick, $message);
}

sub handleMessage {
  my ($this, $irc, $channel, $nick, $message ) = @_;
  if ( $message =~ m@^roll (\d+) d(\d+)\s*$@ ) {
    my $count = $1;
    my $sides = $2;
    
    if ( $count < 1 || $sides < 2 ) {
      $irc->privmsg( $channel, "sorry $nick, i'm bound by the laws of physics" );
    } elsif ( $count > $MAX_COUNT ) {
      $irc->privmsg( $channel, "sorry $nick, i didn't bring that many dice" );
    } elsif ( $sides > $MAX_SIDES ) {
      $irc->privmsg( $channel, "sorry $nick, i left all my big dice at home" );
    } else {
      my @results = roll( $count, $sides );
      my $results = join " ", @results;	
      $irc->privmsg( $channel, "i rolled $results" );
    }
  } elsif ( $message =~ m@^roll soc$@ ) {
    my $count = 3;
    my $sides = 6;
    my ( $normal, $red, $bart ) = roll( $count, $sides );
    $irc->privmsg( $channel, "i rolled $normal \cc04$red\cc " . $ARRGH{$bart} );
  } elsif ( $message =~ m@^roll percentile$@ ) {
    # percentile die is a 00-90 die and a 0-9 die 
    my $count = 2;
    my $sides = 10;
    my ($ones, $tens) = roll( $count, $sides );
    $tens--;
    $tens *= 10;
    $ones--; 
    my $tot = $tens + $ones;
    $tens = sprintf("%02d",$tens);
    $irc->privmsg( $channel, "i rolled $tot ($tens $ones)" ); 
  } elsif ( $message =~ m@^roll roshambo$@ ) {
    # rock paper scissors
    my $count = 1;
    my $sides = 3; 
    my $rps = ("", "rock", "paper", "scissors")[ roll( $count, $sides) ]; 
    if ( rand() < $CARTMAN ) {
      $irc->privmsg( $channel, "i'm gonna kick you in the nuts!" );
    } else {
      $irc->privmsg( $channel, "1... 2... 3... $rps!" );
    }
  } elsif ( $message =~ m@^rps25\s+(\S+)\s+(\S+)\s*$@ ) {
		my $user1 = $1;
		my $user2 = $2;
		my $rps1 = &rps25();
		my $rps2 = &rps25();
                my $victory = &beats($rps1, $rps2);
    $irc->privmsg( $channel, "$user1: $rps1,  $user2: $rps2. $victory" );
  }

}

sub rps25 {
  my $count = 1;
  my $sides = 25; 
  my $rps = ("", 
		"gun",
		"dynamite",
		"nuke",
		"lightning",
		"devil",
		"dragon",
		"alien",
		"water",
		"bowl",
		"air",
		"moon",
		"paper",
		"sponge",
		"wolf",
		"cockroach",
		"tree",
		"man",
		"woman",
		"monkey",
		"snake",
		"axe",
		"scissors",
		"fire",
		"sun",
		"rock",
		)[ roll( $count, $sides) ]; 
	return $rps;
}

# the beat map is composed by taking the (mod) preceeding 12 items in the RPS list above.
# so "gun" beats rock -> wolf. "sponge" beats paper -> gun, etc.
# a more compact version without the custom verbing would take advantage of that mechanic.
my %beat = (
  'gun' => { 'rock' => 'targets',
             'sun'  => 'shoots at',
             'fire' => '',
             'scissors' => 'destroys',
             'axe' => 'chips',
             'snake' => 'shoots',
             'monkey' => 'shoots',
             'woman' => 'shoots',
             'man' => 'shoots',
             'tree' => 'targets',
             'cockroach' => 'shoots',
             'wolf' => 'shoots',
           },
  'dynamite' => {
     'gun' => '>',
     'rock' => '>',
     'sun' => '>',
     'fire' => '>',
     'scissors' => '>',
     'axe' => '>',
     'snake' => '>',
     'monkey' => '>',
     'woman' => '>',
     'man' => '>',
     'tree' => '>',
     'cockroach' => '>',
                },
   'nuke' => {
     'dynamite' => '>',
     'gun' => '>',
     'rock' => '>',
     'sun' => '>',
     'fire' => '>',
     'scissors' => '>',
     'snake' => '>',
     'axe' => '>',
     'monkey' => '>',
     'woman' => '>',
     'man' => '>',
     'tree' => '>',
   },
   'lightning' => {
     'nuke' => '>',
     'dynamite' => '>',
     'gun' => '>',
     'rock' => '>',
     'sun' => '>',
     'fire' => '>',
     'scissors' => '>',
     'axe' => '>',
     'snake' => '>',
     'monkey' => '>',
     'woman' => '>',
     'man' => '>',
   },
   'devil' => {
     'lightning' => 'casts',
     'nuke' => 'inspires',
     'dynamite' => 'inspires',
     'gun' => 'inspires',
     'rock' => 'hurls',
     'sun' => 'curses',
     'fire' => 'breathes',
     'scissors' => 'immune to',
     'axe' => 'immune to',
     'snake' => 'eats',
     'monkey' => 'enrages',
     'woman' => 'tempts',
    },
   'dragon' => {
     'devil' => '>',
     'lightning' => '>',
     'nuke' => '>',
     'dynamite' => '>',
     'gun' => '>',
     'rock' => '>',
     'sun' => '>',
     'fire' => '>',
     'scissors' => '>',
     'axe' => '>',
     'snake' => '>',
     'monkey' => '>',
    },
   'alien' => {
     'dragon' => '>',
     'devil' => '>',
     'lightning' => '>',
     'nuke' => '>',
     'dynamite' => '>',
     'gun' => '>',
     'rock' => '>',
     'sun' => '>',
     'fire' => '>',
     'scissors' => '>',
     'axe' => '>',
     'snake' => '>',
    },
    'water' => {
     'alien' => '>',
     'dragon' => '>',
     'devil' => '>',
     'lightning' => '>',
     'nuke' => '>',
     'dynamite' => '>',
     'gun' => '>',
     'rock' => '>',
     'sun' => '>',
     'fire' => '>',
     'scissors' => '>',
     'axe' => '>',
    },
    'bowl' => {
     'water' => '>',
     'alien' => '>',
     'dragon' => '>',
     'devil' => '>',
     'lightning' => '>',
     'nuke' => '>',
     'dynamite' => '>',
     'gun' => '>',
     'rock' => '>',
     'sun' => '>',
     'fire' => '>',
     'scissors' => '>',
    },
   'air' => {
     'bowl' => '>',
     'water' => '>',
     'alien' => '>',
     'dragon' => '>',
     'devil' => '>',
     'lightning' => '>',
     'nuke' => '>',
     'dynamite' => '>',
     'gun' => '>',
     'rock' => '>',
     'sun' => '>',
     'fire' => '>',
    },
    'moon' => {
     'air' => '>',
     'bowl' => '>',
     'water' => '>',
     'alien' => '>',
     'dragon' => '>',
     'devil' => '>',
     'lightning' => '>',
     'nuke' => '>',
     'dynamite' => '>',
     'gun' => '>',
     'rock' => '>',
     'sun' => '>',
    },
    'paper' => {
     'moon' => '>',
     'air' => '>',
     'bowl' => '>',
     'water' => '>',
     'alien' => '>',
     'dragon' => '>',
     'devil' => '>',
     'lightning' => '>',
     'nuke' => '>',
     'dynamite' => '>',
     'gun' => '>',
     'rock' => '>',
    },
    'sponge' => {
     'paper' => '>',
     'moon' => '>',
     'air' => '>',
     'bowl' => '>',
     'water' => '>',
     'alien' => '>',
     'dragon' => '>',
     'devil' => '>',
     'lightning' => '>',
     'nuke' => '>',
     'dynamite' => '>',
     'gun' => '>',
    },
    'wolf' => {
     'sponge' => '>',
     'paper' => '>',
     'air' => '>',
     'bowl' => '>',
     'water' => '>',
     'alien' => '>',
     'dragon' => '>',
     'devil' => '>',
     'lightning' => '>',
     'nuke' => '>',
     'dynamite' => '>',
    },
    'cockroach' => {
     'wolf' => '>',
     'sponge' => '>',
     'paper' => '>',
     'moon' => '>',
     'air' => '>',
     'bowl' => '>',
     'water' => '>',
     'alien' => '>',
     'dragon' => '>',
     'devil' => '>',
     'lightning' => '>',
     'nuke' => '>',
    },
    'tree' => {
     'cockroach' => '>',
     'wolf' => '>',
     'sponge' => '>',
     'paper' => '>',
     'moon' => '>',
     'air' => '>',
     'bowl' => '>',
     'water' => '>',
     'alien' => '>',
     'dragon' => '>',
     'devil' => '>',
     'lightning' => '>',
    },
    'man' => {
     'tree' => '>',
     'cockroach' => '>',
     'wolf' => '>',
     'sponge' => '>',
     'paper' => '>',
     'moon' => '>',
     'air' => '>',
     'bowl' => '>',
     'water' => '>',
     'alien' => '>',
     'dragon' => '>',
     'devil' => '>',
    },
    'woman' => {
     'man' => '>',
     'tree' => '>',
     'cockroach' => '>',
     'wolf' => '>',
     'sponge' => '>',
     'paper' => '>',
     'moon' => '>',
     'air' => '>',
     'bowl' => '>',
     'water' => '>',
     'alien' => '>',
     'dragon' => '>',
    },
    'monkey' => {
     'woman' => '>',
     'man' => '>',
     'tree' => '>',
     'cockroach' => '>',
     'wolf' => '>',
     'sponge' => '>',
     'paper' => '>',
     'moon' => '>',
     'air' => '>',
     'bowl' => '>',
     'water' => '>',
     'alien' => '>',
    },
    'snake' => {
     'monkey' => '>',
     'woman' => '>',
     'man' => '>',
     'tree' => '>',
     'cockroach' => '>',
     'wolf' => '>',
     'sponge' => '>',
     'paper' => '>',
     'moon' => '>',
     'air' => '>',
     'bowl' => '>',
     'water' => '>',
    },
    'axe' => {
     'snake' => '>',
     'monkey' => '>',
     'woman' => '>',
     'man' => '>',
     'tree' => '>',
     'cockroach' => '>',
     'wolf' => '>',
     'sponge' => '>',
     'paper' => '>',
     'moon' => '>',
     'air' => '>',
     'bowl' => '>',
    },
    'scissors' => {
     'axe' => '>',
     'snake' => '>',
     'monkey' => '>',
     'woman' => '>',
     'man' => '>',
     'tree' => '>',
     'cockroach' => '>',
     'wolf' => '>',
     'sponge' => '>',
     'paper' => '>',
     'moon' => '>',
     'air' => '>',
    },
    'fire' => {
     'scissors' => '>',
     'axe' => '>',
     'snake' => '>',
     'monkey' => '>',
     'woman' => '>',
     'man' => '>',
     'tree' => '>',
     'cockroach' => '>',
     'wolf' => '>',
     'sponge' => '>',
     'paper' => '>',
     'moon' => '>',
    },
    'sun' => {
     'fire' => '>',
     'scissors' => '>',
     'axe' => '>',
     'snake' => '>',
     'monkey' => '>',
     'woman' => '>',
     'man' => '>',
     'tree' => '>',
     'cockroach' => '>',
     'wolf' => '>',
     'sponge' => '>',
     'paper' => '>',
    },
    'rock' => {
     'sun' => '>',
     'fire' => '>',
     'scissors' => '>',
     'axe' => '>',
     'snake' => '>',
     'monkey' => '>',
     'woman' => '>',
     'man' => '>',
     'tree' => '>',
     'cockroach' => '>',
     'wolf' => '>',
     'sponge' => '>',
    },
);

sub beats {
   my ($rps1,$rps2) = @_;
   if ( $rps1 eq $rps2 ) {
     return "a tie! (only 4% chance, whoda thunk)";
   }
   if ( $beat{$rps1}->{$rps2} ) {
     return "$rps1 $beat{$rps1}->{$rps2} $rps2";
   } else {
     return "$rps2 $beat{$rps2}->{$rps1} $rps1";
   }
}


sub roll {
  my($count, $sides) = @_;
  my @f;
  $#f = $count-1;
  return map {int rand($sides)+1} @f;
}

1;
