# $Id: ModRunometer.pm,v 1.2 2007/03/08 21:20:46 uhtu Exp $
# Copyright (c) 2007 Robert Hagemann <bobzilla@gothictech.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModRunometer;

@ModRunometer::ISA = qw(OmniMod);

use strict;

use URI::Escape;

my $on = 1;
my $ua = LWP::UserAgent->new;
$ua->agent('Mozilla/5.0 (Windows; U; Windows NT 5.0; en-US; rv:0.9.2) Gecko/20010726 Netscape6/6.1');
$ua->timeout(10); # 10s timeout

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );
  if ( $message =~ /^runometer\s*$/ ) {
    $irc->privmsg( $channel, &get_runometer_stats( split /\s+/, $message ) );
  } 
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;
  return if ! &applicable( $message );

  if ( $message =~ /^runometer\s*$/ ) {
    $irc->notice( $nick, &get_runometer_stats( split /\s+/, $message ) );
  }
}

sub applicable {
  my $message = shift;
  if ( $message =~ /^runometer\s*$/ ) {
    return 1;
  } 
  return 0;
}

sub get_runometer_stats {
  my @words = @_;

	my $url = 'http://runometer.com/';
  my $request = HTTP::Request->new(GET => $url);
  my $response = $ua->request($request);
  my $html = $response->as_string();
	
	my $retval = "No results";
        # 170 runners. 219 runs, over 790.23 miles. 99 routes.
	if ( $html =~ /([\d,.]+) runners. ([\d,.]+) runs, over ([\d,.]+) miles. ([\d,.]+) routes./si ) {
		my $runners = $1;
                my $runs = $2;
                my $miles = $3;
                my $routes = $4;
                $retval = "$runners runners. $runs runs. $miles miles. $routes routes.";
	}

	return $retval;
}

1;
