# $Id: ModSVN.pm,v 1.6 2006/12/18 18:26:17 bobzilla Exp $
# Copyright (c) 2002-2004 Robert Hagemann <bobzilla@gothictech.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModSVN;

@ModSVN::ISA = qw(OmniMod);

use strict;

use POSIX;
use URI::Escape;

my $on = 1;

my $lastrev;
my $HEADFILE= "/home/rhageman/head.log";
my $postchan = "#dev";
my $waittime = 15;
# boolean to tell whether we were scheduled yet
my $scheduled = 0;

sub help {
  return if ! $on;

  if ( defined wantarray ) { # do we have help?
    return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  
  $irc->notice( $nick, "No help for you!" );
}

sub name() {
  my $this = shift;
  return "svn";
}

sub handleConnect {
  my $this = shift;
  my $irc = shift;

  # schedule the file checking for filetron functionality
  $irc->schedule($waittime, \&checkSVN, $irc);
  $scheduled = 1;
}

sub handleMessage {
  my $this = shift;
  my $irc = shift;

  return if ! $on;

  if ( ! $scheduled ) {
    $irc->schedule($waittime, \&checkSVN, $irc);
    $scheduled = 1;
  }
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

  return if ! $on;

  if ( ! $scheduled ) {
    $irc->schedule($waittime, \&checkSVN, $irc);
    $scheduled = 1;
  }

}

# check svn
sub checkSVN {
	my ($irc) = @_;

	# print "checkSVN\n";

  return if ! -e $HEADFILE;

	my $head = `cat $HEADFILE`;
	$head =~ s/^---.*?---\n//sgi;
	$head =~ s/\n---.*?---\n//sgi;
	$head =~ s/\n/ /sgi;
	$head =~ s/\s* \s*/ /sgi;
	$head =~ s/\s* \s*/ /sgi;
	$head =~ s/(lines?)/$1 | /si;
	
	$head =~ /^\s*(r\d+)\s+/si;
	my $rev = $1;
	
	if ( ! defined $rev ) {
		print STDERR "head: $head rev: '$rev'\n";
	}

	if ( defined $lastrev and $lastrev ne $rev and $head !~ /^\s*$/si ) {
	  # print "print: $head\n";
	  $irc->privmsg( $postchan, $head );
		&ModChatLog::saveMessage('omnicron', $head, $postchan );
	}

	if ( defined $rev ) {
		$lastrev = $rev;
	}

	# reschedule
	$irc->schedule($waittime, \&checkSVN);
}

1;
