# $Id: ModSeen.pm,v 1.1 2006/12/21 04:20:28 bobzilla Exp $
# Copyright (c) 2003-2004 Robert Hagemann <bobzilla@gothictech.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

package ModSeen;

@ModSeen::ISA = qw(OmniMod);

use strict;

my $on = 1;

my $table = "SEENTABLE";

sub help {
  return if ! $on;

  if ( defined wantarray ) { # do we have help?
    return 1;
  }

  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;

	$irc->notice( $nick, "[seen <nick>] when was nick last seen" );
}

sub name() {
  my $this = shift;
  return "seen";
}

sub handleMessage {
  my $this = shift;
  my $irc = shift;
  my $channel = shift;
  my $nick = shift;
  my $message = shift;
  my $event = shift;

  return if ! $on;

  my $output = &checkAnyMessage( $irc, $nick, $message, $channel, 0 );
	&ModChatLog::saveMessage('omnicron', $output, $channel )
    if defined $output and $output !~ /^\s*$/si;
}

sub handlePrivateMessage {
  my $this = shift;
  my $irc = shift;
  my $nick = shift;
  my $message = shift;
  my $event = shift;

  return if ! $on;

  &checkAnyMessage( $irc, $nick, $message, $nick, 1 );
}

sub checkAnyMessage {
  my $irc = shift;
  my $nick = shift;
  my $message = shift;
  my $sendTo = shift;
  my $isPrivate = shift;

  my $output;

  if ( ! $isPrivate and $message !~ /^\s*[%]/si ) {
    my $now = localtime( time );
    &getData( $nick, "$nick was last seen $now saying \"$message\"" );
  }

  if ( $message =~ /^\s*seen\s+(\S+)\s*$/si ) {
    my $seennick = $1;

    my $old = &getData( $seennick );
    if ( defined $old ) {
      $output = $old;
    }
    else {
      $output = "i haven't seen $seennick";
    }
  }

  $irc->privmsg( $sendTo, $output ) if defined $output;

  return $output;
}

sub saying {
  my $thing = shift;
  my $old = shift;
  my $nick = shift;
  my $isPrivate = shift;

  my $output;

	my $rand = int (rand 6);
	$output = "i heard $thing is $old" if $rand == 0;
	$output = "like, $thing is, like, so $old" if $rand == 1;
	$output = "$thing is $old" if $rand == 2;
	$output = "someone said $thing is $old" if $rand == 3;
	$output = "$thing is $old, $nick" if $rand == 4;
  # a chance to do nothing publically
	$output = "$thing is $old, $nick" if $rand == 5 and $isPrivate;

  return $output;
}

sub getData {
  my $name = shift;
  my $add = shift;

	my $data = OmniHash::getHash( $table );
  $$data{ $name } = $add if defined $add;
	delete $$data{ $name } if defined $add and $add eq "";
  my $retval = $$data{ $name };
	OmniHash::closeAllHashes();

  return $retval;
}

1;
